ChatWheel = LibStub("AceAddon-3.0"):NewAddon("ChatWheel", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub ("AceLocale-3.0"):GetLocale("ChatWheel", true)

-- ----------------
--      GUI      --
-- ----------------
ChatWheel.GUI = {}

function ChatWheel.GUI:SetFrame(frame, w, h, bgFile, edgeFile, edgeSize, BCR, BCG, BCB, BCA, BBCR, BBCG, BBCB, BBCA)
    frame:SetSize(w, h)
    frame:SetBackdrop({
        bgFile = bgFile,
        insets = {left = 1,right = 1,top = 1,bottom = 1},
        edgeFile = edgeFile,
        edgeSize = edgeSize,
    })
    frame:SetBackdropColor(BCR, BCG, BCB, BCA)
    frame:SetBackdropBorderColor(BBCR, BBCG, BBCB, BBCA)
end

function ChatWheel.GUI:CreateWheelBackground()
    local wheelBG = CreateFrame("Frame", "ChatWheelFrameBG", UIParent)
    ChatWheel.GUI:SetFrame(wheelBG, UIParent:GetWidth(), UIParent:GetHeight(), "Interface\\AddOns\\ChatWheel\\media\\normTex", "Interface\\AddOns\\ChatWheel\\media\\normTex", 0, 0, 0, 0, 0, 0, 0, 0, 0)
    wheelBG:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
    wheelBG:EnableMouse(true)
    wheelBG:SetScript("OnMouseUp", function(self, button)
        if button == "RightButton" then
            ChatWheel.GUI.wheelBG:Hide()
        elseif button == "LeftButton" then
            ChatWheel.GUI:onSelectText(degree)
        end
    end)
    ChatWheel.GUI.wheelBG = wheelBG
    tinsert(UISpecialFrames, "ChatWheelFrameBG");
end


function ChatWheel.GUI:CreateWheel()
    ChatWheel.GUI:CreateWheelBackground()
    local mouseX, mouseY = GetCursorPosition()
    local mouseX = mouseX/UIParent:GetEffectiveScale()
    local mouseY = mouseY/UIParent:GetEffectiveScale()

    local wheelFrame = CreateFrame("Frame", "ChatWheelFrame", ChatWheel.GUI.wheelBG)
    ChatWheel.GUI:SetFrame(wheelFrame, 100, 100, "Interface\\AddOns\\ChatWheel\\media\\normTex", "Interface\\AddOns\\ChatWheel\\media\\normTex", 1, 0, 0, 0, 0, 0, 0, 0, 0)
    wheelFrame:SetPoint("CENTER", ChatWheel.GUI.wheelBG, "BOTTOMLEFT", mouseX, mouseY)

    local centerIcon = CreateFrame("Frame", "ChatWheelCenterIcon", wheelFrame)
    ChatWheel.GUI:SetFrame(centerIcon, 24, 24, "Interface\\AddOns\\ChatWheel\\media\\circle", "Interface\\AddOns\\ChatWheel\\media\\normTex", 1, 0.1, 0.1, 0.1, 1, 0, 0, 0, 0)
    centerIcon:SetPoint("CENTER", wheelFrame, "CENTER", 0, 0)
    wheelFrame.centerIcon = centerIcon


    local arrowIcon = CreateFrame("Frame", "ChatWheelArrow", wheelFrame)
    ChatWheel.GUI:SetFrame(arrowIcon, 6, 6, "Interface\\AddOns\\ChatWheel\\media\\circle", "Interface\\AddOns\\ChatWheel\\media\\normTex", 0.9, 0.9, 0.9, 1, 1, 0, 0, 0, 0)
    arrowIcon:SetPoint("CENTER", wheelFrame, "CENTER", 16, 0)
    wheelFrame.arrowIcon = arrowIcon

    wheelFrame.Texts = {}
    local Xs = {-42.5,  -30,   0,     30,   42.5,  30,   0,     -30}
    local Ys = {0,      30,    42.5,  30,   0,     -30,  -42.5, -30}
    local Anchors = {"RIGHT", "RIGHT", "BOTTOM", "LEFT", "LEFT", "LEFT", "TOP", "RIGHT"}
    for i = 1, 8, 1 do
        wheelFrame.Texts[i] = CreateFrame("Frame", "ChatWheelText"..i, wheelFrame)
        ChatWheel.GUI:SetFrame(wheelFrame.Texts[i], 3, 3, "Interface\\AddOns\\ChatWheel\\media\\normTex", "Interface\\AddOns\\ChatWheel\\media\\normTex", 1, 0.1, 0.1, 0.1, 1, 0, 0, 0, 0)
        wheelFrame.Texts[i]:SetPoint("CENTER", wheelFrame.centerIcon, "CENTER", Xs[i], Ys[i])
        wheelFrame.Texts[i].Text = wheelFrame.Texts[i]:CreateFontString(nil, "OVERLAY")
        wheelFrame.Texts[i].Text:SetFont(STANDARD_TEXT_FONT, 14, "THINOUTLINE")
        if ChatWheel.Args.db.global.MessageTable[i] then
            wheelFrame.Texts[i].Text:SetText(ChatWheel.Args.db.global.MessageTable[i])
        end
        wheelFrame.Texts[i].Text:SetShadowColor(0, 0, 0, 0.8)
        wheelFrame.Texts[i].Text:SetShadowOffset(2, -2)
        wheelFrame.Texts[i].Text:SetPoint(Anchors[i], wheelFrame.Texts[i], "CENTER", 0, 0)
        wheelFrame.Texts[i].agFlag = 0
    end
    wheelFrame.Texts.lastAGFlag = nil

    ChatWheel.GUI.wheelFrame = wheelFrame
    ChatWheel.GUI.wheelBG:Hide()

end


function ChatWheel:parseMessage(msg)
    local i = 1
    local key, value = nil, nil
    for v in string.gmatch(msg, "([^..%-..]+)") do
        if i == 1 then
            key = v
        elseif i == 2 then
            value = v
        end
        i = i+ 1
     end
     return key, value
end

function ChatWheel.GUI:SetWheelMessage(i)
    if ChatWheel.Args.db.global.MessageTable[i] then
        local k, v = ChatWheel:parseMessage(ChatWheel.Args.db.global.MessageTable[i])
        ChatWheel.GUI.wheelFrame.Texts[i].Text:SetText(k)
    else
        ChatWheel.GUI.wheelFrame.Texts[i].Text:SetText(L["Empty"])
    end
end


function ChatWheel.GUI:CalcMouseAngle()
    local point, relativeTo, relativePoint, xOfs, yOfs = ChatWheel.GUI.wheelFrame:GetPoint()
    local realWidth = UIParent:GetWidth() * UIParent:GetEffectiveScale()
    local realHeight = UIParent:GetHeight() * UIParent:GetEffectiveScale()

    local mouseX, mouseY = GetCursorPosition()
    mouseX = mouseX/UIParent:GetEffectiveScale()
    mouseY = mouseY/UIParent:GetEffectiveScale()

    local offX, offY = xOfs - mouseX, yOfs - mouseY
    local sin = offY / math.sqrt(offX*offX + offY*offY)
    local cos = offX / math.sqrt(offX*offX + offY*offY)
    local sinDegree = asin(sin)
    local cosDegree = acos(cos)
    local degree = 0
    if sinDegree > 0 then
        degree = 360 - cosDegree
    else
        degree = cosDegree
    end
    return cos, sin, degree
end

function ChatWheel.GUI:EnlargeTextByDegree(degree)
    local degreeIdx = floor((degree + 22.5) / 45) + 1
    if degreeIdx == 9 then degreeIdx = 1 end
    if not ChatWheel.GUI.wheelFrame.Texts.lastAGFlag then
        ChatWheel.GUI.wheelFrame.Texts.lastAGFlag = degreeIdx
        ChatWheel.GUI.wheelFrame.Texts[degreeIdx].agFlag = 1
        ChatWheel.GUI.wheelFrame.Texts[ChatWheel.GUI.wheelFrame.Texts.lastAGFlag].Text:SetScale(1.5, 1.5)
    else
        if ChatWheel.GUI.wheelFrame.Texts.lastAGFlag == degreeIdx then
            return
        else
            ChatWheel.GUI.wheelFrame.Texts[ChatWheel.GUI.wheelFrame.Texts.lastAGFlag].agFlag = 0
            ChatWheel.GUI.wheelFrame.Texts[ChatWheel.GUI.wheelFrame.Texts.lastAGFlag].Text:SetScale(1, 1)
            ChatWheel.GUI.wheelFrame.Texts.lastAGFlag = degreeIdx
            ChatWheel.GUI.wheelFrame.Texts[degreeIdx].agFlag = 1
            ChatWheel.GUI.wheelFrame.Texts[ChatWheel.GUI.wheelFrame.Texts.lastAGFlag].Text:SetScale(1.5, 1.5)
        end
    end
end


function ChatWheel.GUI:BindMouseMove()
    ChatWheel.GUI.wheelFrame:SetScript("OnUpdate", function(self)
        -- Calculate Angle
        local cos, sin, degree = ChatWheel.GUI:CalcMouseAngle()
        ChatWheel.GUI.mouseDegree = degree
        -- Animation
        ChatWheel.GUI.wheelFrame.arrowIcon:SetPoint("CENTER", ChatWheel.GUI.wheelFrame, "CENTER", -16*cos, -16*sin)
        ChatWheel.GUI:EnlargeTextByDegree(degree)

    end)
end


function ChatWheel.GUI:onSelectText()
    local degreeIdx = floor((ChatWheel.GUI.mouseDegree + 22.5) / 45) + 1
    if degreeIdx == 9 then degreeIdx = 1 end
    local success = ChatWheel:PrintMessage(degreeIdx)
    if success then
        ChatWheel.GUI.wheelBG:Hide()
    end
end

-- ----------------
--      ARGS     --
-- ----------------

ChatWheel.Args = {}
function ChatWheel.Args.Initialize()
    ChatWheel.Args.db = LibStub("AceDB-3.0"):New("ChatWheelArgs", {
        global = {
            MessageTableStr = L["Save Me"].."-"..L["Save Me"].."\r\n"..L["Stay Here"].."-"..L["Stay Here"].."\r\n"..L["Wait"].."-"..L["Wait"].."\r\n"..L["Time Warp Now"].."-"..L["Time Warp Now"].."\r\n"..L["Two Awak"].."-"..L["Two Awak"].."\r\n"..L["Away"].."-"..L["Away"].."\r\n"..L["Pull a Big Wave"].."-"..L["Pull a Big Wave"].."\r\n"..L["Ceb"].."-"..L["Ceb"],
            MessageTable = {},
            Channel = "SAY",
            bindedkey = "END",
            SavedTable = {},
            SavedTableKeys = {},
        },
    })
end


-- ----------------
--     Print     --
-- ----------------

function ChatWheel:PrintMessage(idx)
    local msg = ChatWheel.Args.db.global.MessageTable[idx]
    local channel = ChatWheel.Args.db.global.Channel
    if msg then
        local k, v = ChatWheel:parseMessage(msg)
        SendChatMessage(v, channel, "Common");
        return true
    else
        return false
    end
end


function ChatWheel:OnInitialize()
    ChatWheel.Args.Initialize()
    ChatWheel:Print("Initial")
    ChatWheel.GUI:CreateWheel()
    ChatWheel.GUI:BindMouseMove()
    SetBinding(ChatWheel.Args.db.global.bindedkey, "SHOW_WHELL");
    ChatWheel:setMessageTable(nil, ChatWheel.Args.db.global.MessageTableStr)
    ChatWheel.optionsTable.args.group1.args.selectSaved.values = ChatWheel.Args.db.global.SavedTableKeys
end


ChatWheel:RegisterChatCommand("CW", "CommandHandler")

function ChatWheel:CommandHandler(input)
    ChatWheel:Print(input)
end

